@@ -2,9 +2,6 @@
 use std::fmt;
 use std::cmp;
 
-#[cfg(feature = "use-serde")]
-use serde::{Serialize, Deserialize};
-
 pub mod compare_tensor;
 pub mod convolution;
 pub mod elemwise;
@@ -13,11 +10,14 @@ pub mod linalg;
 pub mod reduction;
 pub mod rand;
 
+#[cfg(feature = "use-serde")]
+use serde::{Serialize, Deserialize};
+
 /// Naive tensor implementation, single thread
 #[cfg_attr(feature = "use-serde", derive(Serialize, Deserialize))]
 pub struct GenTensor<T> {
-    d: Vec<T>,
-    dim: Vec<usize>,
+    d: Vec<T>, // GenTensor is stored in row major, 
+    dim: Vec<usize>, // higher/outer dimension is on the lower index.
 }
 impl<T> GenTensor<T> where T: num_traits::Float {
 
@@ -206,6 +206,8 @@ impl<T> GenTensor<T> where T: num_traits::Float {
         Ok(())
     }
 
+    /// stride() returns a vector contains steps
+    /// for each dimension on linear storage.
     /// Right dimension changes fastest.
     /// Right dimension has the stride 1.
     ///
@@ -741,7 +743,7 @@ impl<T> GenTensor<T> where T: num_traits::Float {
     }
 
     pub fn log10_like(&self) -> GenTensor<T> {
-	let new_data = vec![T::from(2.302585092994046).unwrap(); self.d.len()];
+	let new_data = vec![T::from(std::f64::consts::LN_10).unwrap(); self.d.len()];
         let new_dim = self.dim.to_vec();
         GenTensor {
             d: new_data,
@@ -749,7 +751,7 @@ impl<T> GenTensor<T> where T: num_traits::Float {
         }
     }
     pub fn log2_like(&self) -> GenTensor<T> {
-	let new_data = vec![T::from(0.6931471805599453).unwrap(); self.d.len()];
+	let new_data = vec![T::from(std::f64::consts::LN_2).unwrap(); self.d.len()];
         let new_dim = self.dim.to_vec();
         GenTensor {
             d: new_data,
@@ -769,7 +771,7 @@ impl<T> GenTensor<T> where T: num_traits::Float {
     /// ```
     pub fn add(&self, o: &GenTensor<T>) -> GenTensor<T> {
         self._right_broadcast(o, |x, y| *x + *y)
-     }
+    }
     pub fn sub(&self, o: &GenTensor<T>) -> GenTensor<T> {
         self._right_broadcast(o, |x, y| *x - *y)
     }
@@ -839,7 +841,7 @@ impl<T> GenTensor<T> where T: num_traits::Float {
     /// matrix multiplication of two tensor
     /// This is also for dot/inner product.
     pub fn matmul(&self, o: &GenTensor<T>) -> GenTensor<T> {
-        if self.dim[self.dim.len()-1] != o.dim[0] {
+        if self.size()[self.size().len()-1] != o.size()[0] {
             panic!("matmul expect matched size {:?}, {:?}", self.dim, o.dim);
         }
         if self.size().len() == 1 && o.size().len() == 1 {
@@ -945,12 +947,9 @@ impl<T> GenTensor<T> where T: num_traits::Float {
                 dim,
             }
         }
-        
-        
-        
     }
 
-    #[cfg(not(feature = "use-blas"))]
+    #[cfg(not(feature = "use-blas-lapack"))]
     pub fn squared_error(t1: &Self, t2: &Self) -> GenTensor<T> {
         let mut ret = GenTensor {
             d: Vec::with_capacity(t1.d.len()),
@@ -1227,7 +1226,10 @@ impl fmt::Display for GenTensor<f64> {
 
 impl fmt::Debug for GenTensor<f32> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        if self.dim.len() == 2 {
+        if self.numel() > 30 {
+            writeln!(f, "size: {:?}", self.dim)?;
+            write!(f, "data: {:?}", &self.d[..30])
+        } else if self.dim.len() == 2 {
             write!(f, "[")?;
             for i in 0..self.dim[0] {
                 write!(f, "[")?;
@@ -1245,7 +1247,10 @@ impl fmt::Debug for GenTensor<f32> {
 }
 impl fmt::Debug for GenTensor<f64> {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        if self.dim.len() == 2 {
+        if self.numel() > 30 {
+            writeln!(f, "size: {:?}", self.dim)?;
+            write!(f, "data: {:?}", &self.d[..30])
+        } else if self.dim.len() == 2 {
             write!(f, "[")?;
             for i in 0..self.dim[0] {
                 write!(f, "[")?;
@@ -1271,11 +1276,13 @@ impl<T> Clone for GenTensor<T> where T: num_traits::Float {
     }
 }
 
-#[cfg(feature = "use-blas")]
-use crate::tensor_impl::lapack_tensor::blas::BlasAPI;
+#[cfg(feature = "use-blas-lapack")]
+use crate::tensor_impl::lapack_tensor::blas_api::BlasAPI;
 
-#[cfg(feature = "use-blas")]
+
+#[cfg(feature = "use-blas-lapack")]
 impl GenTensor<f32> {
+    
     pub fn squared_error(t1: &Self, t2: &Self) -> GenTensor<f32> {
         let mut v2 = t2.d.to_vec();
         BlasAPI::<f32>::axpy(t1.d.len(), -1., &t1.d, 1, &mut v2, 1);
@@ -1290,8 +1297,9 @@ impl GenTensor<f32> {
         ret
     }
 }
-#[cfg(feature = "use-blas")]
+#[cfg(feature = "use-blas-lapack")]
 impl GenTensor<f64> {
+    
     pub fn squared_error(t1: &Self, t2: &Self) -> GenTensor<f64> {
         let mut v2 = t2.d.to_vec();
         BlasAPI::<f64>::axpy(t1.d.len(), -1., &t1.d, 1, &mut v2, 1);
@@ -1462,22 +1470,4 @@ mod tests {
         let c = a.ne(&b);
         assert_eq!(c, GenTensor::<f32>::new_raw(&vec![1., 0., 0., 1.], &vec![2,2]));
     }
-    
-
-}
-
-
-#[cfg(all(test, feature = "use-serde"))]
-mod test_for_serde {
-    use super::*;
-
-    #[test]
-    fn test_serde() {
-        let m1 = GenTensor::<f32>::new_raw(&vec![1.,2.,3.,4.,5.,6.], &vec![3,2]);
-
-        let serialized = serde_pickle::to_vec(&m1, true).unwrap();
-        let deserialized = serde_pickle::from_slice(&serialized).unwrap();
-        //println!("{:?}", deserialized);
-        assert_eq!(m1, deserialized);
-    }
 }
